% Genetic Algorithm
% Function 'genetic' is created in this program to find the 
% optimum parameters for the system whose fitness function
% is stored in 'objfunction'. For a parameter optimization problem,
% functions 'genetic' and 'objfunction' must first be created as m-files

% INPUT VARIABLES
% popsize: Population size (even integer)
% objfunction: Function to be maximized; a valid existing M-file
% N: number of variables in objfunction
% minmax: Nx2 matrix; jth row contains minimum and maximum values of the
% jth variable
% nbit: 1xN row vector; jth element contains number of bits 
% for coding jth variable
% pc: Probability of cross over (pc<1)
% pm : Probability of mutation (pm<1)
% maxg : Maximum number of generations
% OUTPUT VARIABLE
% opt_param : Optimal parameters

function [opt_param]=genetic(popsize,objfunc,N,minmax,nbit,pc,pm,maxg)

% Initialize generation counter, 'gc', and stopping flag 'stop'.

gc=1;
stop=0;

% Select random initial decimal values of parameters in the
% popsizexN matrix (refer Note 1)

for i=1:popsize
    for j=1:N
        inivar(i,j)=round(rand*(2^nbit(j)-1));
    end
end

% Convert decimal values to binary bits and generate
% popsizexlchrom bitmatrix where lchrom is the length 
% of a chromosome(refer Note 2)

lchrom=sum(nbit);

bitmat=zeros(popsize,lchrom);

for i=1:popsize
    k=lchrom;
    for j=N:-1:1
        x=inivar(i,j);
        for m=1:nbit(j)
            bitmat(i,k)=rem(x,2);
            x=fix(x/2);
            k=k-1;
        end
    end
end

% MAIN LOOP STARTS HERE

while (stop==0)
    gc
    % Calculate actual parameter values from bitmatrix 
    % refer ( Note 3)
    for j=1:N
        for i=1:popsize
            param(i,j)=minmax(j,1)...
                +(minmax(j,2)-minmax(j,1))/...
                (2^nbit(j)-1)*inivar(i,j);
        end
    end
    
    % Calculate fitness values
    sumfit=0;
    for i=1:popsize
        argu=param(i,1:N);
        fitness(i)=feval(objfunc,argu);
        sumfit=sumfit+fitness(i);
    end
    avgfit=sumfit/popsize;
    
    % Parameters giving max fitness are given out as
    % optimum parameters
    
    [maxfit,index]=max(fitness);
    opt_param=param(index,:);
    % fit_ratio to be used in termination criteria
    fit_ratio=avgfit/maxfit;
    % Build an array of avg & max fitness values and
    % record them generation-wise
    af(gc)=avgfit;
    mf(gc)=maxfit;
    
    % Generate mating pool through Roulette wheel selection
    
    sum1=0;
    
    for i=1:popsize
        %Compute propability of selection of the ith chromosome
        ps=fitness(i)/sumfit;
        % Obtain cumulative sum of the selection probabilities
        sum1=sum1+ps;
        csum(i)=sum1;
    end
    
    % randperm returns random sequencing of numbers from 1 to
    % popsize
    
    mplocation=randperm(popsize);
    
    % The randomly spun RW selects jth chromosome & places it at
    % a random location in mating pool(refer Note 4)
    
    for i=1:popsize
        rwspin=rand;
        for j=1:popsize
            if(rwspin<=csum(j))
                for k=1:lchrom
                    mpool(mplocation(i),k)=bitmat(j,k);                    
                end
                break;
            else
            end
        end
    end
    
    % Crossover operator
    % Number of crossovers nc
    nc=round(pc*popsize/2);
    %Increment by 2 since crossover is pairwise (refer Note 5)
    for i=1:2:nc
        xsite=round(rand*lchrom);
        for j=1:xsite
            temp=mpool(i,j);
            mpool((i+1),j)=temp;
        end
    end
    
    
    % Mutation operator
    % Number of mutations nm
    nm=round(pm*lchrom*popsize);
    
    while(nm>0)
        for i=1:nm
            msite=round(rand*lchrom*popsize);
            % msite is being worked out not just within a
            % particular chromosome but over the entire bitmatrix
            if(msite==0)
                msite=1;
                % since nm>0, make the 0 msite as 1 so that atleast
                % one mutation does take place
            end

            % Location of the bit to be mutated

            colm=rem(msite,lchrom);
            if (colm==0)
                colm=lchrom;
                row=fix(msite/lchrom);
            else
                row=fix(msite/lchrom)+1;
            end

            if (mpool(row,colm)==0)
                mpool(row,colm)=1;
            else
                mpool(row,colm)=0;
            end

        end
        nm=0;
    end
    
    % Decoding
    % Identify the bits in a string corresponding to a
    % parameter (refer Note 6)
    
    for i=1:popsize
        b1=0;
        bn=0;
        for j=1:N
            sum1=0;
            n=nbit(j);
            if (j==1)
                b1=1;
            else
                b1=b1+nbit(j-1);
            end
            bn=bn+nbit(j);
            % Calculate the Decimal value
            for k=b1:bn
                n=n-1;
                sum1=sum1+(mpool(i,k))*(2^n);
            end
            var(i,j)=sum1;
        end
    end
    
    
    % Increment the generation counter
    gc=gc+1;
    inivar=var;
    bitmat=mpool;
    
    % Stopping criteria
    
    if((gc>maxg)|(fit_ratio>0.9999))
        stop=1;
    else
        stop=0;
    end
    
    
end

% MAIN LOOP  ENDS

if (gc > maxg)
    disp('GOAL NOT REACHED');
end

kf=1:gc-1;
plot(kf,mf,'b',kf,af,'r')
legend('maxfit','avgfit')


    
     
    
    
       
    





